#!/usr/bin/env python

# Copyright (C) 2012-2013 Richard Sartor <richard.sartor@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import subprocess
import socket
import logging
import threading
import Queue
import time
import requests

from . import __version__


logger = logging.getLogger(__name__)


_default_jar_file = 'usr/share/repose/repose-valve.jar'


class ReposeValve:
    def __init__(self, config_dir, port=None, https_port=None, jar_file=None,
                 stop_port=None, insecure=False, wait_on_start=False):
        logger.debug('Creating new ReposeValve object (config_dir=%s, '
                     'jar_file=%s, stop_port=%s, insecure=%s)' %
                     (config_dir, jar_file, stop_port, insecure))

        if jar_file is None:
            jar_file = _default_jar_file

        if stop_port is None:
            if port is None:
                stop_port = 9090
            else:
                stop_port = port + 1000

        if wait_on_start:
            if port is not None:
                wait_url = 'http://localhost:%s/' % str(port)
            elif https_port is not None:
                wait_url = 'https://localhost:%s' % str(https_port)
            else:
                raise ValueError("Either 'port' and/or 'https_port' must "
                                 "specify a port number if 'wait_on_start' is "
                                 "True")

        self.config_dir = config_dir
        self.port = port
        self.jar_file = jar_file
        self.stop_port = stop_port
        self.insecure = insecure

        pargs = [
            'java', '-jar', jar_file,
            '-c', config_dir,
            '-s', str(stop_port)
        ]

        if port is not None:
            pargs.append('-p')
            pargs.append(str(port))

        if https_port is not None:
            pargs.append('-ps')
            pargs.append(str(https_port))

        if insecure:
            pargs.append('-k')

        pargs.append('start')

        logger.debug('Starting valve with the following commonad line: "%s"' %
                     ' '.join(pargs))

        self.proc = subprocess.Popen(pargs, stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
        self.stdout = ThreadedStreamReader(self.proc.stdout)
        self.stderr = ThreadedStreamReader(self.proc.stderr)

        if wait_on_start:
            wait_count = 0
            while True:
                try:
                    sc = requests.get(wait_url)
                    if int(sc) == 200:
                        break
                except:
                    pass
                time.sleep(1)
                wait_count += 1
                if wait_count > 30:
                    break

        logger.debug('New ReposeValve object initialized (pid=%i)' %
                     self.proc.pid)

    def stop(self, wait=True):
        try:
            logger.debug('Shutting down stdout and stderr readers.')
            self.stdout.shutdown()
            self.stderr.shutdown()

            logger.debug('Attempting to stop ReposeValve object (pid=%i, '
                         'stop_port=%s)' % (self.proc.pid, self.stop_port))
            s = socket.create_connection(('localhost', self.stop_port))
            s.send('stop\r\n')
            if wait:
                logger.debug('Waiting for process to end (pid=%i)' %
                             self.proc.pid)
                self.wait()
        except:
            logger.debug('Couldn\'t stop using the stop port, killing instead '
                         '(pid=%i)' % self.proc.pid)
            self.proc.kill()
        logger.debug('Repose stopped (pid=%i)' % self.proc.pid)

    def wait(self):
        return self.proc.communicate()


class ThreadedStreamReader:
    def __init__(self, stream):
        self.stream = stream
        self.thread = threading.Thread(target=self._thread_target)
        self.thread.daemon = True
        self.thread.start()
        self.queue = Queue.Queue()
        self._shutdown = False

    def _thread_target(self):
        for line in self.stream.xreadlines():
            if self._shutdown:
                break
            self.queue.put(line)

    def readline(self, timeout=None):
        s = self.queue.get(timeout=timeout)
        self.queue.task_done()
        return s

    def readlines(self):
        lines = []
        while not self.queue.empty():
            lines.append(self.readline())
        return lines

    def shutdown(self):
        self._shutdown = True


def stream_printer(fin, fout):
    while True:
        for line in fin.readlines():
            fout.write(line)
            fout.flush()
        time.sleep(1)



#!/usr/bin/env python

import argparse
import time
import repose

from . import __version__


def run():

    parser = argparse.ArgumentParser()
    parser.add_argument('config_dir',
                        help='The location of the Repose config directory.')
    parser.add_argument('--port', type=int,
                        help='The port on which Repose will listen for '
                        'requests.')
    parser.add_argument('--https-port', type=int,
                        help='The port on which Repose will listen for '
                        'requests.')
    parser.add_argument('--stop-port', type=int, default=None,
                        help='The port on which Repose will listen for the '
                        'shutdown command. Default is PORT+1000.')
    parser.add_argument('--insecure', help='Don\'t verify SSL certs.',
                        action='store_true')
    parser.add_argument('--jar-file', help='The Repose Valve JAR file to run.',
                        default=repose._default_jar_file)

    args = parser.parse_args()

    if args.stop_port is None:
        if args.port is None:
            args.stop_port = 9090
        else:
            args.stop_port = int(args.port) + 1000

    r = repose.ReposeValve(jar_file=args.jar_file, config_dir=args.config_dir,
                           port=args.port, https_port=args.https_port,
                           stop_port=args.stop_port, insecure=args.insecure)

    r.wait()


if __name__ == '__main__':
    run()



#!/usr/bin/env python

import inspect
import os
import string
import xml.etree.ElementTree as et

from . import __version__


def join_path(a, *p):
    return os.path.normpath(os.path.join(a, *p))


def create_folder(folder_name):
    if not os.path.exists(folder_name):
        os.makedirs(folder_name)


def get_configs_folder():
    _script_filename = os.path.abspath(inspect.getfile(inspect.currentframe()))
    _script_folder = os.path.dirname(_script_filename)
    configs_folder = '%s/configs' % _script_folder
    return configs_folder


def get_config_sets(configs_folder):
    if not os.path.exists(configs_folder) or not os.path.isdir(configs_folder):
        return
    for entry in os.listdir(configs_folder):
        if os.path.isdir('%s/%s' % (configs_folder, entry)):
            if os.path.exists(join_path(configs_folder, entry,
                                        '.config-set.xml')):
                yield entry


def process_config_set(config_set_name, destination_path=None,
                       configs_folder=None, params=None, verbose=True):

    if params is None:
        params = {}

    if os.path.isfile(config_set_name):
        # it's a file
        config_xml = et.parse(config_set_name)
        source_context = os.path.dirname(config_set_name)
    else:
        # try a named config set in the configs folder
        if configs_folder is None:
            configs_folder = get_configs_folder()
        if config_set_name not in get_config_sets(configs_folder):
            raise NamedConfigSetNotFoundException(config_set_name)
        filename = join_path(configs_folder, config_set_name,
                             '.config-set.xml')
        config_xml = et.parse(filename)
        source_context = join_path(configs_folder, config_set_name)

    for folder in config_xml.findall('folder'):
        folder_path = folder.attrib.get('path', '.')
        for f in folder.findall('file'):
            file_source = join_path(source_context, f.attrib['src'])
            file_basename = os.path.basename(file_source)
            if destination_path and folder_path:
                full_dest = join_path(destination_path, folder_path)
            elif destination_path:
                full_dest = destination_path
            elif folder_path:
                full_dest = folder_path
            else:
                full_dest = '.'
            create_folder(full_dest)
            file_dest = join_path(full_dest, file_basename)

            if verbose:
                applying = ''
                if len(params) > 0:
                    # TODO: maybe output parameters provided/substituted?
                    applying = ', applying config parameters'

                print ('Copy from "%s" to "%s"%s' %
                       (file_source, file_dest, applying))

            copy_and_apply_params(file_source, file_dest, params, verbose)


class NamedConfigSetNotFoundException(Exception):
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return "No config set named \"%s\" was found." % self.name


def copy_and_apply_params(source, dest, params={}, verbose=True):
    with open(source, 'r') as input:
        template = string.Template(input.read())

    with open(dest, 'w') as output:
        subst = template.safe_substitute(params)
        unsubst = template.pattern.findall(subst)
        if verbose:
            for match in unsubst:
                name = match[1] or match[2] or None
                if name is not None:
                    print ("Warning: Unsubstituted value \"%s\" in template." %
                           name)
        output.write(subst)



#!/usr/bin/env python

import argparse
from conf import (NamedConfigSetNotFoundException, process_config_set,
                  get_configs_folder, get_config_sets)


def run():
    parser = argparse.ArgumentParser(description='Copy config files',
                                     version='1.0')
    parser.add_argument('--param', action='append',
                        help='A name/value pair for substitution of template '
                        'parameters.')
    parser.add_argument(metavar='config-set', dest='config_set',
                        help='A set of configuration files to copy. Must be a '
                        'named config set in the configs/ folder.')
    parser.add_argument('--dest-path', dest='dest_path',
                        help='Where to put the files.')
    args = parser.parse_args()

    params = {}
    if args.param is not None:
        for param in args.param:
            parts = param.split('=', 2)
            if len(parts) > 1:
                name, value = parts
            else:
                name, value = parts[0], 'true'
            params[name] = value
    args.params = params

    configs_folder = get_configs_folder()

    config_set = args.config_set
    dest_path = args.dest_path

    try:
        process_config_set(config_set_name=config_set,
                           configs_folder=configs_folder, params=params,
                           destination_path=dest_path)

    except NamedConfigSetNotFoundException as e:
        print 'Error: %s' % str(e)
        found = False
        for cs in get_config_sets(configs_folder):
            if not found:
                print 'Available config sets:'
            print '  %s' % cs
            found = True
        if not found:
            print 'No available config sets found'


if __name__ == '__main__':
    run()



#!/usr/bin/env python

import re
import logging
import requests
import xml.etree.ElementTree as et
import os
import sys

from . import __version__


logger = logging.getLogger(__name__)


class MavenConnector(object):
    def get_artifact_url(self, root, extension, snapshot=False, version=None):

        meta = '%s/maven-metadata.xml' % root
        metas = requests.get(meta).text
        metax = et.fromstring(metas)
        artifact_id = metax.find('artifactId').text
        if snapshot:
            if version is None:
                version = metax.find('versioning/latest').text
                m = re.match('(\d+\.\d+\.\d+)-SNAPSHOT', version)
                main_version = m.group(1)
                snapshot_version = None
            else:
                version = str(version)
                m = re.match('(\d+\.\d+\.\d+)(-SNAPSHOT|$)', version)
                if m is not None:
                    main_version = m.group(1)
                    snapshot_version = None
                else:
                    m = re.match('(\d+\.\d+\.\d+)-(\d+\.\d+-\d+)', version)
                    if m is None:
                        raise Exception('Invalid version format: "%s"' %
                                        version)
                    main_version = m.group(1)
                    snapshot_version = '%s-%s' % (main_version, m.group(2))
                found = False
                for vv in metax.findall('versioning/versions/version'):
                    # kludge - searching through the list because ET doesn't
                    # have complete xpath predicate support
                    if vv.text == '%s-SNAPSHOT' % main_version:
                        found = True
                        break
                if not found:
                    raise Exception('Version "%s" not found in the metadata' %
                                    main_version)
            version_root = '%s/%s-SNAPSHOT' % (root, main_version)
            meta2 = '%s/maven-metadata.xml' % version_root
            meta2s = requests.get(meta2).text
            meta2x = et.fromstring(meta2s)
            if snapshot_version is None:
                last_updated = meta2x.find('versioning/lastUpdated').text
                for elem in meta2x.findall('versioning/snapshotVersions/'
                                           'snapshotVersion'):
                    if (elem.find('extension').text == extension and
                            (not elem.find('classifier') or
                             elem.find('classifier').text != 'sources') and
                            elem.find('updated').text == last_updated):
                        snapshot_version = elem.find('value').text
            else:
                found = False
                for elem in meta2x.findall('versioning/snapshotVersions/'
                                           'snapshotVersion'):
                    if (elem.find('extension').text == extension and
                            elem.find('value').text == snapshot_version):
                        found = True
                if not found:
                    raise Exception('Snapshot version "%s" not found in the '
                                    'metadata' % (snapshot_version))
            artifact_url = '%s/%s-%s.%s' % (version_root, artifact_id,
                                            snapshot_version, extension)
            return artifact_url
        else:
            if version is None:
                version = metax.find('versioning/release').text
            else:
                version = str(version)
                found = False
                for vv in metax.findall('versioning/versions/version'):
                    # kludge - searching through the list because ET doesn't
                    # have complete xpath predicate support
                    if vv.text == version:
                        found = True
                        break
                if not found:
                    raise Exception('Version "%s" not found in the metadata' %
                                    version)
            version_root = '%s/%s' % (root, version)
            artifact_url = '%s/%s-%s.%s' % (version_root, artifact_id, version,
                                            extension)
            return artifact_url

        return None

    def clean_up_dest(self, url, dest=None):
        """Clean up the destination. If dest is None, use the filename of the
        file being downloaded and store it in the current directory. If dest
        ends with a '/', or if it points to a directory, use the filename of
        the file being downloaded and store it in the specified directory. If
        dest doesn't end with a '/' and doesn't point to a directory, store
        the file using the specfied filename. If the specified already exists,
        append a number to it."""
        if dest == '' or dest is None:
            dest = os.path.basename(url)
        else:
            logger.debug('cleaning up dest')
            logger.debug('dest: %s' % dest)
            logger.debug('os.path.isdir(dest): %s' % os.path.isdir(dest))
            basename = os.path.basename(dest)
            dirname = os.path.dirname(dest)
            if os.path.isdir(dest) or basename == '':
                basename = os.path.basename(url)
                dirname = dest
            else:
                basename = os.path.basename(dest)
                dirname = os.path.dirname(dest)
            logger.debug('basename: %s' % basename)
            logger.debug('dirname: %s' % dirname)
            basename = os.path.normpath(basename)
            dirname = os.path.normpath(dirname)
            logger.debug('basename: %s' % basename)
            logger.debug('dirname: %s' % dirname)
            logger.debug('os.path.exists(dirname): %s' %
                         str(os.path.exists(dirname)))
            if dirname != '' and os.path.exists(dirname):
                n = 1
                basename2 = basename
                logger.debug('basename2: %s' % basename2)
                logger.debug('os.path.exists(join(dirname, basename2)): %s'
                             % os.path.exists(os.path.join(dirname,
                                                           basename2)))
                while os.path.exists(os.path.join(dirname, basename2)):
                    basename2 = basename + '.%i' % n
                    n += 1
                    logger.debug('basename2: %s' % basename2)
                    logger.debug('os.path.exists(os.path.join(dirname, '
                                 'basename2)): %s' %
                                 os.path.exists(os.path.join(dirname,
                                                             basename2)))
                basename = basename2
            dest = os.path.join(dirname, basename)
            logger.debug('dest [final]: %s' % dest)
        return dest

    def download_file(self, url, dest):

        response = requests.get(url, stream=True)
        if response.status_code != 200:
            raise ValueError

        dirname = os.path.dirname(dest)
        if dirname != '' and not os.path.exists(dirname):
            os.makedirs(dirname)

        blocksize = 4096
        count = 0
        with open(dest, 'wb') as f:
            for block in response.iter_content(blocksize):
                f.write(block)
                count += len(block)
                if count > 100000:
                    count = 0
                    sys.stdout.write('.')
                    sys.stdout.flush()
            print



#!/usr/bin/env python

import pyrax
import os
import argparse
import time
import paramiko
import getpass
import pushy

from . import __version__


def create_server(credential_file=None, username=None, api_key=None,
                  image=None, flavor=None, server_name=None,
                  server_name_prefix=None):
    """Create a Server."""

    if not (username or api_key or credential_file):
        raise ValueError("No credentials specified")
    if credential_file and (username or api_key):
        raise ValueError("Conflicting credential options specified.")
    if api_key and not username:
        raise ValueError("No username specified")
    if username and not api_key:
        raise ValueError("No API key specified")

    # TODO: get credentials from command line
    if credential_file:
        pyrax.set_credential_file(credential_file)
    else:
        pyrax.set_credentials(username=username, api_key=api_key)

    cs = pyrax.cloudservers

    if image is None:
        image = [img for img in cs.images.list()
                 if "CentOS 6.3" in img.name][0]
    if flavor is None:
        flavor = [fl for fl in cs.flavors.list() if fl.ram == 1024][0]

    if server_name is None:
        t = time.localtime()
        date_string = '%d-%02d-%02d-%02d-%02d-%02d' % (t.tm_year, t.tm_mon,
                                                       t.tm_mday, t.tm_hour,
                                                       t.tm_min, t.tm_sec)
        whoami = getpass.getuser()
        if server_name_prefix is None:
            server_name_prefix = 'repose'
        server_name = '%s-%s-%s' % (server_name_prefix, date_string, whoami)

    server = cs.servers.create(name=server_name,
                               image=image, flavor=flavor)

    print time.asctime()
    n = 0
    while n < 600:
        server2 = cs.servers.get(server.id)
        if server2.status == u'ACTIVE':
            break
        if server2.status != u'BUILD' and server2.status != u'UNKNOWN':
            raise Exception("Server entered an invalid state: '%s'" %
                            server.status)
        time.sleep(15)
        print '%s - %s (%s%%)' % (time.asctime(), server2.status,
                                  str(server2.progress))
        n += 15

    if server2.status != u'ACTIVE':
        raise Exception('Server failed to build in time')

    server2.adminPass = server.adminPass
    return server2


def add_server_to_known_hosts_file(server):
    ips = server.networks['public']
    os.system('ssh-keyscan %s 2>/dev/null >> ~/.ssh/known_hosts' %
              ' '.join(ips))


def install_prereqs(server):
    username = 'root'
    password = server.adminPass
    ips = server.networks['public']

    ssh_client = paramiko.SSHClient()
    ssh_client.load_system_host_keys()
    server.ssh_client = ssh_client

    connected = False

    for ip in ips:
        try:
            ssh_client.connect(hostname=ip, username=username,
                               password=password)
            connected = True
            break
        except:
            pass

    if not connected:
        raise Exception("Could not connect")

    def readfile(file):
        try:
            return file.read()
        except:
            return None

    def exec_command2(command, bufsize=-1):
        return map(readfile, ssh_client.exec_command(command, bufsize))

    ssh_client.exec_command2 = exec_command2

    epel_filename = 'epel-release-6-8.noarch.rpm'
    epel_url = ('http://download.fedoraproject.org/pub/epel/6/i386/%s' %
                epel_filename)

    stdio = exec_command2('wget %s' % epel_url)
    stdio = exec_command2('rpm -Uvh %s' % epel_filename)
    stdio = exec_command2('rm -f %s' % epel_filename)
    stdio = exec_command2('yum install -y gcc')
    stdio = exec_command2('yum install -y python python-pip')
    stdio = exec_command2('yum install -y git')
    stdio = exec_command2('yum install -y java-1.6.0-openjdk')
    stdio = exec_command2('pip-python install virtualenv')
    stdio = exec_command2('PIP=`which pip-python` ;'
                          ' ln -s $PIP `dirname $PIP`/pip')

    stdio = exec_command2('git clone '
                          'git://github.com/richard-sartor/narwhal.git')
    stdio = exec_command2('virtualenv narwhal')
    stdio = exec_command2('cd narwhal ; . bin/activate ;'
                          ' pip install -r requirements.txt')

    conn = pushy.connect(target='ssh:%s' % ip,
                         python='/root/narwhal/bin/python', username='root',
                         password=server.adminPass)
    conn.modules.os.chdir('/root/narwhal')
    server.pushy = conn
    return conn


def install_repose(server):
    # tell the remote server to download the most recent snapshot artifacts
    # from Nexus
    #
    return server.pushy.modules.install_repose.get_repose()


def run_remote_repose(server, config_dir, stop_port, **kwargs):
    if config_dir is None:
        config_dir = 'etc/repose'
    return server.pushy.modules.repose.ReposeValve(config_dir=config_dir,
                                                   stop_port=7777, **kwargs)


def open_iptables_port(server, port):
    return server.ssh_client.exec_command2('iptables -I INPUT -m state '
                                           '--state NEW -m tcp -p tcp --dport '
                                           '%s -j ACCEPT' % str(port))


def create_rserver(config_dir, port, credential_file=None, username=None,
                   api_key=None, image=None, flavor=None, server_name=None,
                   server_name_prefix=None):
    server = create_server(credential_file=credential_file,
                           username=username, api_key=api_key,
                           image=image, flavor=flavor,
                           server_name_prefix=server_name_prefix)
    add_server_to_known_hosts_file(server)
    install_prereqs(server)
    downloads = install_repose(server)

    params = {
        'port': port,
        'deploydir': 'var/repose',
        'artifactdir': 'usr/share/repose/filters',
        'logfile': 'var/log/repose/current.log',
    }
    open_iptables_port(server, port)
    server.pushy.modules.conf.process_config_set(config_set_name='simple-node',
                                                 destination_path=config_dir,
                                                 params=params, verbose=False)
    server.ssh_client.exec_command('mkdir -p ~/narwhal/var/repose')
    repose = run_remote_repose(server, config_dir=config_dir, stop_port=7777,
                               wait_on_start=True, port=port)
    server.repose = repose
    return server


def run():
    parser = argparse.ArgumentParser(description="Create a server and install "
                                     "Repose on it.")
    parser.add_argument('--credential-file',
                        help="Specify credentials to use to authenticate to "
                        "the compute service")
    parser.add_argument('--username')
    parser.add_argument('--api-key')
    parser.add_argument('--image')
    parser.add_argument('--flavor')
    parser.add_argument('--name-prefix')

    args = parser.parse_args()

    if args.credential_file and (args.username or args.api_key):
        raise ValueError("Conflicting credential options specified.")
    if args.api_key and not args.username:
        raise ValueError("No username specified")
    if not (args.username or args.api_key or args.credential_file):
        raise ValueError("No credentials specified")
    if args.username and not args.api_key:
        raise ValueError("No API key specified")

    server = create_rserver(credential_file=args.credential_file,
                            username=args.username, api_key=args.api_key,
                            image=args.image, flavor=args.flavor,
                            server_name_prefix=args.name_prefix,
                            config_dir='etc/repose', port=11111)


if __name__ == '__main__':
    run()



#!/usr/bin/env python

import os
import shutil

from . import __version__


def delete_folder(folder_name):
    if os.path.exists(folder_name):
        shutil.rmtree(folder_name)


def create_folder(folder_name):
    if not os.path.exists(folder_name):
        os.makedirs(folder_name)


def clear_folder(folder_name):
    for _name in os.listdir(folder_name):
        name = join(folder_name, _name)
        if os.path.isdir(name):
            delete_folder(name)
        else:
            os.remove(name)


def join(a, *p):
    return os.path.normpath(os.path.join(a, *p))



#!/usr/bin/env python

import argparse
import logging
import maven

from . import __version__


logger = logging.getLogger(__name__)


class ReposeMavenConnector(maven.MavenConnector):
    _default_url_root = ('http://maven.research.rackspacecloud.com/'
                         'content/repositories')
    _default_valve_dest = 'usr/share/repose/repose-valve.jar'
    _default_filter_dest = 'usr/share/repose/filters/filter-bundle.ear'
    _default_ext_filter_dest = ('usr/share/repose/filters/'
                                'extensions-filter-bundle.ear')

    def get_repose_valve_url(self, root, snapshot=False, version=None):
        if snapshot:
            s_or_r = 'snapshots'
        else:
            s_or_r = 'releases'

        vroot = "%s/%s/com/rackspace/papi/core/valve" % (root, s_or_r)

        return self.get_artifact_url(vroot, 'jar', snapshot=snapshot,
                                     version=version)

    def get_filter_bundle_url(self, root, snapshot=False, version=None):
        if snapshot:
            s_or_r = 'snapshots'
        else:
            s_or_r = 'releases'

        froot = ('%s/%s/com/rackspace/papi/components/filter-bundle' %
                 (root, s_or_r))

        f_artifact_url = self.get_artifact_url(froot, 'ear', snapshot=snapshot,
                                               version=version)

        return f_artifact_url

    def get_extensions_filter_bundle_url(self, root, snapshot=False,
                                         version=None):
        if snapshot:
            s_or_r = 'snapshots'
        else:
            s_or_r = 'releases'

        eroot = ("%s/%s/com/rackspace/papi/components/extensions/"
                 "extensions-filter-bundle" % (root, s_or_r))

        e_artifact_url = self.get_artifact_url(eroot, 'ear', snapshot=snapshot,
                                               version=version)

        return e_artifact_url

    def get_repose(self, url_root=None, valve_dest=None, filter_dest=None,
                   ext_filter_dest=None, get_valve=True, get_filter=True,
                   get_ext_filter=True, snapshot=False, version=None):

        if url_root is None:
            url_root = ReposeMavenConnector._default_url_root
        if valve_dest is None:
            valve_dest = ReposeMavenConnector._default_valve_dest
        if filter_dest is None:
            filter_dest = ReposeMavenConnector._default_filter_dest
        if ext_filter_dest is None:
            ext_filter_dest = ReposeMavenConnector._default_ext_filter_dest

        if get_valve:
            vurl = self.get_repose_valve_url(root=url_root, snapshot=snapshot,
                                             version=version)
        if get_filter:
            furl = self.get_filter_bundle_url(root=url_root, snapshot=snapshot,
                                              version=version)
        if get_ext_filter:
            eurl = self.get_extensions_filter_bundle_url(root=url_root,
                                                         snapshot=snapshot,
                                                         version=version)

        filenames = {}

        if get_valve:
            valve_dest = self.clean_up_dest(vurl, valve_dest)
            print '%s --> %s' % (vurl, valve_dest)
            if vurl:
                self.download_file(url=vurl, dest=valve_dest)
                filenames["valve"] = valve_dest

        if get_filter:
            filter_dest = self.clean_up_dest(furl, filter_dest)
            print '%s --> %s' % (furl, filter_dest)
            if furl:
                self.download_file(url=furl, dest=filter_dest)
                filenames["filter"] = filter_dest

        if get_ext_filter:
            ext_filter_dest = self.clean_up_dest(eurl, ext_filter_dest)
            print '%s --> %s' % (eurl, ext_filter_dest)
            if eurl:
                self.download_file(url=eurl, dest=ext_filter_dest)
                filenames["ext_filter"] = ext_filter_dest

        return filenames


def run():

    parser = argparse.ArgumentParser()
    parser.add_argument('--valve-dest', help='The name that the Valve JAR '
                        'should be renamed to, or the directory where it '
                        'should be downloaded to.')
    parser.add_argument('--filter-dest', help='The name that the filter '
                        'bundle EAR file should be renamed to, or the '
                        'directory where it should be downloaded to.')
    parser.add_argument('--ext-filter-dest', help='The name that the '
                        'extension filter bundle EAR file should be renamed '
                        'to, or the directory where it should be downloaded '
                        'to.')
    parser.add_argument('--no-valve',
                        help='Don\'t download the valve JAR file',
                        action='store_true')
    parser.add_argument('--no-filter', help='Don\'t download the standard '
                        'filter bundle EAR file', action='store_true')
    parser.add_argument('--no-ext-filter', help='Don\'t download the '
                        'extension filter bundle EAR file',
                        action='store_true')
    parser.add_argument('--url-root', help='The url (with path) to download '
                        'artifacts from.')
    parser.add_argument('--snapshot', help='Download a SNAPSHOT build instead '
                        'of a release build.', action='store_true')
    parser.add_argument('--version', help='The version of the artifacts to '
                        'download. Typically of the forms "x.y.z" for '
                        'releases, "x.y.z-SNAPSHOT" for the most recent '
                        'snapshot build in version x.y.z, and '
                        '"x.y.z-date.time-build" for a specific snapshot '
                        'build.', type=str)
    parser.add_argument('--print-log', help="Print the log to STDERR.",
                        action='store_true')
    parser.add_argument('--full-log', help="Log more information.",
                        action='store_true')
    args = parser.parse_args()

    if args.print_log:
        if args.full_log:
            logging.basicConfig(level=logging.DEBUG,
                                format='%(levelname)s:%(name)s:%(funcName)s:'
                                '%(filename)s(%(lineno)d):%(threadName)s'
                                '(%(thread)d):%(message)s')
        else:
            logging.basicConfig(level=logging.DEBUG)

    rmc = ReposeMavenConnector()
    rmc.get_repose(url_root=args.url_root, valve_dest=args.valve_dest,
                   filter_dest=args.filter_dest,
                   ext_filter_dest=args.ext_filter_dest,
                   get_valve=not args.no_valve, get_filter=not args.no_filter,
                   version=args.version, get_ext_filter=not args.no_ext_filter,
                   snapshot=args.snapshot)


if __name__ == '__main__':
    run()



#!/usr/bin/env python

from setuptools import setup
import narwhal

setup(
    name='narwhal',
    version=narwhal.__version__,
    packages=['narwhal'],
    license='MIT License',
    long_description=open('README.txt').read(),
    author='Richard Sartor',
    author_email='richard.sartor@rackspace.com',
    url='https://github.com/richard-sartor/narwhal',
    description='Setup scripts for repose',
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
    install_requires=[
        'requests',
        'argparse',
        'pushy',
        'pyrax',
        'paramiko',
    ],
    entry_points={
        'console_scripts': [
            'download-repose = narwhal.download_repose:run',
            'run-repose = narwhal.run_repose:run',
            'configure-repose = narwhal.configure_repose:run',
        ],
    }
)
